package alice.tucsonx.sms;

import alice.logictuple.*;
import alice.tucson.api.*;
import java.util.*;

/**
 * La classe rappresenta l'agente AlarmClock.
 *
 * @author Mattia Bargellini
 * @version 1.0
 */
public class AlarmClock extends Agent {

	/**Id per il tc*/
	private TupleCentreId tid;

	/**Vettore dinamico usato nel metodo body*/
	private Vector v;

	/**
	 * Costruttore dell'agente AlarmClock.
	 *
	 * @param id     Id dell'agente
	 * @param target Id del centro di tuple su cui l'agente nasce
 	 */
	public AlarmClock(AgentId id, TupleCentreId target)
		throws TucsonException {
		super(id);
		tid = target;
		v = new Vector();
	}

	/**
	 * Corpo dell'agente
 	 */
	protected void body() {

		try {

			while (true) {

				int[] array_date = new int[5];

				while(v.size() == 0) {

					/*Creo il template della tupla da leggere*/
					LogicTuple tpl =
						new LogicTuple(
							"when",
							new Var("Id"),
							new Var("Date"),
							new Var("Time"));

					/*Eseguo la in da tc*/
					LogicTuple when = in(tid, tpl);

					/*Estraggo i valori dalla tupla*/
					long id = when.getArg(0).longValue();
					String date = when.getArg(1).toString();
					String time = when.getArg(2).toString();

					array_date = createDate(date,time);

					System.out.println("\nNew Service:");
					System.out.println("Date & Time:");
					System.out.println("day:"+array_date[0]);
					System.out.println("month:"+array_date[1]+"-->"+calculateMonth(array_date[1]));
					System.out.println("year:"+array_date[2]);
					System.out.println("hour:"+array_date[3]);
					System.out.println("min:"+array_date[4]);

					/*Se c' un errore nella data/ora si inserisce una tupla di errore nel tid, altrimenti si
					controlla il timestamp con quello corrente*/
					if(array_date[0]<1 || array_date[0]>31 || array_date[1]<0 ||
						array_date[1]>11 || array_date[3]<0 || array_date[3]>24
						|| array_date[4]<0 || array_date[4]>59) {

						LogicTuple error =
									new LogicTuple(
										"error",
										new Value(id),
										new Value("error in date or time format"));
						System.out.println("Found error in date or time format. Tuple id:"+id);
						out(tid, error);
					} else {

						/*Estraggo la data di invio dai valori letti nella tupla*/
						GregorianCalendar c = new GregorianCalendar();
						/*NB:indice array: 0-->day, 1-->month, 2-->year, 3-->hour, 4-->min*/
						c.set(array_date[2],array_date[1],array_date[0],array_date[3],array_date[4]);
						long timestamp =c.getTime().getTime();

						/*ricalcolo il timestamp corrente
						(la in  bloccante e l'attesa per una nuova tupla potrebbe essere anche
						piuttosto lunga)*/
						long current_timestamp = (new Date()).getTime();

//System.out.println("timestamp:"+timestamp+"\ntimestamp corrente:"+current_timestamp);

						if(timestamp == current_timestamp){

							LogicTuple wake_up =
									new LogicTuple(
									"wake_up",
									new Value(id));
							System.out.println("Wake up tuple with id:"+id);
							out(tid, wake_up);
						} else {

							/*Se il timestamp  > di quello corrente lo inserisco insieme id nello stesso
							vector che viene controllato all'inizio del metodo body
							Altrimenti scrivo una tupla di errore*/
							if(timestamp > current_timestamp){
								Vector v2 = new Vector(2);

								Long lid = Long.valueOf(Long.toString(id));
								Long ltimestamp = Long.valueOf(Long.toString(timestamp));
								v2.add(0,lid);
								v2.add(1,ltimestamp);
//System.out.println("vettore v2:"+v2.toString());
								v.addElement(v2);
								System.out.println("Tuple stored (id:"+lid+")");
//System.out.println("vettore v:"+v.toString());
							} else {

								LogicTuple error =
											new LogicTuple(
											"error",
											new Value(id),
											new Value("timestamp older than current timestamp"));
								System.out.println("Found error: timestamp older than current timestamp. Tuple id:"+id);
								out(tid, error);
							}

						 }

					}//if
				}//while con vettore vuoto

				/*Se il vettore non  vuoto, si controlla che
				il timestamp presente nel vettore v sia <= a quello corrente.
				Se lo  allora viene scritta la tupla wake_up nel tid e si cancella l'elemento
				dal vettore*/
				while(v.size()>0) {

					try{
						/*Creo il template della tupla da leggere*/
						LogicTuple tpl =
							new LogicTuple(
								"when",
								new Var("Id"),
								new Var("Date"),
								new Var("Time"));

						/*Eseguo la inp da tc*/
						LogicTuple when = inp(tid, tpl);

						/*Estraggo i valori dalla tupla*/
						long id = when.getArg(0).longValue();
						String date = when.getArg(1).toString();
						String time = when.getArg(2).toString();

						array_date = createDate(date,time);

						System.out.println("\nNew Service:");
						System.out.println("Date & Time:");
						System.out.println("day:"+array_date[0]);
						System.out.println("month:"+array_date[1]+"-->"+calculateMonth(array_date[1]));
						System.out.println("year:"+array_date[2]);
						System.out.println("hour:"+array_date[3]);
						System.out.println("min:"+array_date[4]);

						/*Se c' un errore nella data/ora si inserisce una tupla di errore nel tid,
						altrimenti si controlla il timestamp con quello corrente*/
						if(array_date[0]<1 || array_date[0]>31 || array_date[1]<0 ||
							array_date[1]>11 || array_date[3]<0 || array_date[3]>24
							|| array_date[4]<0 || array_date[4]>59) {

							LogicTuple error =
									new LogicTuple(
										"error",
										new Value(id),
										new Value("error in date or time format"));
							System.out.println("Found error in date or time format. Tuple id:"+id);
							out(tid, error);
						} else {

							/*Estraggo la data di invio dai valori letti nella tupla*/
							GregorianCalendar c = new GregorianCalendar();
							/*NB:indice array: 0-->day, 1-->month, 2-->year, 3-->hour, 4-->min*/
							c.set(array_date[2],array_date[1],array_date[0],array_date[3],array_date[4]);
							long timestamp =c.getTime().getTime();

							/*ricalcolo il timestamp corrente
							(la in  bloccante e l'attesa per una nuova tupla potrebbe essere anche
							piuttosto lunga)*/
							long current_timestamp = (new Date()).getTime();

//System.out.println("timestamp:"+timestamp+"\ntimestamp corrente:"+current_timestamp);

							if(timestamp == current_timestamp){

								LogicTuple wake_up =
										new LogicTuple(
										"wake_up",
										new Value(id));
								System.out.println("Wake up tuple with id:"+id);
								out(tid, wake_up);
							} else {

								/*Se il timestamp  > di quello corrente lo inserisco insieme id
								nello stesso
								vector che viene controllato all'inizio del metodo body
								Altrimenti scrivo una tupla di errore*/
								if(timestamp > current_timestamp){
									Vector v2 = new Vector(2);

									Long lid = Long.valueOf(Long.toString(id));
									Long ltimestamp = Long.valueOf(Long.toString(timestamp));
									v2.add(0,lid);
									v2.add(1,ltimestamp);
//System.out.println("vettore v2:"+v2.toString());
									v.addElement(v2);
									System.out.println("Tuple stored (id:"+lid+")");
//System.out.println("vettore v:"+v.toString());

								} else {

									LogicTuple error =
												new LogicTuple(
												"error",
												new Value(id),
												new Value("timestamp older than current timestamp"));
									System.out.print("Found error:");
									System.out.print(" timestamp older than current timestamp.");
									System.out.print("Tuple id:"+id);
									out(tid, error);
								}

						 	}

						}

					} catch(Exception ex) {
						//System.out.println("An exception has occurred: " + ex);
						//ex.printStackTrace();
						//System.out.println("No tuple to read in tuple centre "+tid+". Go on checking old tuples...");
					}

//System.out.println("v.size:"+v.size());

					//timestamp corrente
					long current_tmstmp = (new Date()).getTime();
//System.out.println("\ncurrent_timestamp:"+current_tmstmp+"\n");

					for(int i=0;i<=v.size()-1;i++) {
//System.out.println("i:"+i);
         				Vector v1 = (Vector) v.elementAt(i);

         				Long l = (Long) v1.elementAt(0);;
						long xid = l.longValue();
//System.out.println("xid:"+xid);
						Long l1 = (Long) v1.elementAt(1);
						long tmstmp = l1.longValue();
//System.out.println("tmstamp:"+tmstmp+"\n");

						if(tmstmp <= current_tmstmp) {
							LogicTuple wake_up =
								new LogicTuple(
									"wake_up",
									new Value(xid));
							System.out.println("Wake up tuple with id:"+xid);
							out(tid, wake_up);

							//elimino l'elemento dal vettore
							v.removeElementAt(i);
						}
					}//for
				}//while vettore NON vuoto

			}//while
		} catch (Exception ex) {
			System.out.println("An exception has occurred: " + ex);
			ex.printStackTrace();
		}
	}//body

	/**
	 * Estrae la data sotto forma di valori interi
	 *
	 * @param date rappresenta la stringa della data
	 * @param time rappresenta la stringa dell'ora
	 *
	 * @return un array di interi rispettivamente giorno, mese, anno, ora e minuti
	 */
	private int[] createDate(String date, String time) {

			int[] ar = new int[5];
			try{

				/*Elimino gli apici dalle stringhe date e time*/
				String newdate = date.substring(1, date.length() - 1);
				String newtime = time.substring(1, time.length() - 1);

				/*Struttura di date: dd-mm-yyyy.
				Estraggo giorno mese e anno come int*/
				String day = newdate.substring(0,2);
				ar[0] = Integer.valueOf(day).intValue();

				String month = newdate.substring(3,5);
				ar[1] = Integer.valueOf(month).intValue() - 1; //Gennaio  il mese 0

				String year = newdate.substring(6,10);
				ar[2] = Integer.valueOf(year).intValue();

				/*struttura di time: hh:mm (l'ora  nel formato 24ore)*/
				String hour = newtime.substring(0,2);
				ar[3] = Integer.valueOf(hour).intValue();

				String min = newtime.substring(3,5);
				ar[4] = Integer.valueOf(min).intValue();

			} catch(NumberFormatException ex) {
				System.out.println("An exception has occurred: " + ex);
				ex.printStackTrace();
			}
			return ar;
	}//createDate

	/**
	 * Calcola il mese
	 *
	 * @param month il numero del mese
	 *
	 * @return il nome del mese
	 */
	private String calculateMonth(int month){
		String m="";
		switch(month) {
		case 0: m = "January";break;
		case 1: m = "February";break;
		case 2: m = "March";break;
		case 3: m = "April";break;
		case 4: m = "May";break;
		case 5: m = "June";break;
		case 6: m = "July";break;
		case 7: m = "August";break;
		case 8: m = "September";break;
		case 9: m = "October";break;
		case 10: m = "November";break;
		case 11: m = "December";break;
		default: m = "This month does not exist";break;
		}
		return m;
	}

}//AlarmClock
