package alice.tucsonx.sms;

import alice.logictuple.*;
import alice.tucson.api.*;
import java.util.*;
/**
 * La classe rappresenta l'agente Checker
 * Questo agente  generato dal tuple center nella reazione settata all'avvio del nodo
 *
 * @author Mattia Bargellini
 * @version 1.0
 */

public class Checker extends Agent {

	/**Lunghezza del numero del cellulare (comprensivo del prefisso internazionale)*/
	private final static int PHONE_NUMBER_LENGTH= 13;

	/**Lunghezza del testo*/
	private final static int TEXT_LENGTH = 8;

	private TupleArgument tpl;
	/**Centro di tuple di default*/
	private final static String DEFAULT_TID = "sms";
	private TupleCentreId tid;

	/**
	 * Costruttore della classe
	 */
	public Checker (AgentId id, LogicTuple args) throws Exception{
			super(id);
			//System.out.println(""+args);
			tpl = args.getArg(0);
			System.out.println("Starting Checker...");
		}

	/**Corpo*/
	protected void body() {

			try {

				TupleCentreId tid = new TupleCentreId(DEFAULT_TID);

				/*Estraggo i valori dalla tupla*/
				String id = tpl.getArg(0).toString();
				List recipient = tpl.getArg(1).toList();
				String text = tpl.getArg(2).toString();

				System.out.println("Checker is working...");
				System.out.println("Id: "+id);

				String[] rec_without_marks = new String[recipient.size()];
				for(int index=0;index<recipient.size();index++) {
					rec_without_marks[index] = recipient.get(index).toString().substring(1, 14);
					System.out.println("Recipient: "+rec_without_marks[index]);
				}

				System.out.println("Text: "+text);

				/*Elimino le apici dal testo*/
				String text_without_marks = removeQuotationMarks(text);

				int k=0;
//System.out.println("size"+recipient.size());
				if(recipient.size()==1) {
					if (!isNumberCorrect(rec_without_marks[0])) {
							// fake service
							/*Creo il template della tupla da scrivere*/
							LogicTuple towrite =
									new LogicTuple(
										"sms_service_status",
										new Value(id),
										new Value("Null service"));
							System.out.println("Incorrect Recipient.Service status: null service");
							/*Eseguo la out sul tc*/
							out(tid, towrite);
					} else {
						if(text_without_marks.length()<=TEXT_LENGTH){
							LogicTuple towrite =
								new LogicTuple(
										"sms_to_send",
										new Value(id),
										new Value(rec_without_marks[0]),
										new Value(text_without_marks));

							System.out.println("Writing tuple: "+towrite);
							out(tid, towrite);
						} else {

							String id1= "0";
							int nro_blocchi = howManyBlocks(text_without_marks);

							String[] subtext = new String[nro_blocchi+1];

							/*Creo n tuple*/
							for(int i=1; i<=nro_blocchi; i++) {
								//Struttura nuovo id: id.xx.yy
								String id2 = ""+id+"."+id1+"."+i;

								if(i==nro_blocchi) {
									/*Splitto i messaggi*/
									subtext[i] = text_without_marks.substring(TEXT_LENGTH*(i-1));
								} else {
									/*Splitto i messaggi*/
									subtext[i] = text_without_marks.substring(TEXT_LENGTH*(i-1),
																				TEXT_LENGTH*i);
								}

								LogicTuple towrite =
									new LogicTuple(
										"sms_to_send",
											new Value(id2),
												new Value(rec_without_marks[0]),
												new Value(subtext[i]));

								System.out.println("Writing tuple: "+towrite);
								out(tid, towrite);
							}//for
						}
					}
				} else {
					while(k<recipient.size()) {

						//Struttura nuovo id; id.xx
						String id1 = id+"."+k;

						if (!isNumberCorrect(rec_without_marks[k])) {
							// fake service
							/*Creo il template della tupla da scrivere*/
							LogicTuple towrite =
									new LogicTuple(
										"sms_service_status",
										new Value(id1),
										new Value("Null service"));
							System.out.println("Incorrect Recipient.Service status: null service");
							/*Eseguo la out sul tc*/
							out(tid, towrite);
						} else {

						   /* Se il testo  piu' corto di TEXT_LENGTH creo tante tuple
						 	* quanti sono i destinatari del messaggio
						 	*
						 	* Se il testo  piu' lungo di TEXT_LENGTH inizialmente "splitto"
						 	* la tupla poi, per ogni testo splittato,
							* creo tante tuple quanti sono i destinatari del messaggio
						 	*/
							if(text_without_marks.length()<=TEXT_LENGTH) {
								LogicTuple towrite =
									new LogicTuple(
											"sms_to_send",
											new Value(id1),
											new Value(rec_without_marks[k]),
											new Value(text_without_marks));

								System.out.println("Writing tuple: "+towrite);
								out(tid, towrite);
							} else {

								int nro_blocchi = howManyBlocks(text_without_marks);

								String[] subtext = new String[nro_blocchi+1];

								/*Creo n tuple*/
								for(int i=1; i<=nro_blocchi; i++) {
									//Struttura nuovo id: id.xx.yy
									String id2 = ""+id1+"."+i;

									if(i==nro_blocchi) {
										/*Splitto i messaggi*/
										subtext[i] = text_without_marks.substring(TEXT_LENGTH*(i-1));
									} else {
										/*Splitto i messaggi*/
										subtext[i] = text_without_marks.substring(TEXT_LENGTH*(i-1),
																				TEXT_LENGTH*i);
									}

									LogicTuple towrite =
											new LogicTuple(
												"sms_to_send",
												new Value(id2),
												new Value(rec_without_marks[k]),
												new Value(subtext[i]));

									System.out.println("Writing tuple: "+towrite);
									out(tid, towrite);
								}//for esterno

							}//if
						}//if
						k++;
					}//while
				}//if
		} catch(Exception ex) {
			System.out.println("An exception has occurred: " + ex);
			ex.printStackTrace();
		}
	}//body

	/**Elimina gli apici dal testo solo se la lunghezza della stringa  maggiore di 4
	 *@param txt stringa da cui rimuovee gli apici
	 *@return la stringa senza gli apici*/
	private String removeQuotationMarks(String txt) {
		String newtxt = "";
		if(txt.length()>4)
		{return newtxt = txt.substring(1, txt.length()-1);}
		else
		{return newtxt = txt;}
	}//removeQuotationMarks

	/**Calcola da quanti blocchi di 160 caratteri  formato Text
	 *@param text il testo da analizzare
	 *@return il numero di blocchi*/
	private int howManyBlocks(String txt) {
		int blocchi = 0;
		float x = (float) txt.length()/TEXT_LENGTH;

		/*Parte frazionaria di x*/
		int tronc = (int) x;

		/*Se la parte frazionaria di x  <0.5 aggiungo 1, altrimenti no.*/
		if(x-tronc<0.5) {return blocchi = Math.round(x)+1;}
		else {return blocchi = Math.round(x);}
	}//howManyBlocks


	/*calcola quanti sono i destinatari
	private String[] recipients(String rec) {
		int count = 0;

		System.out.println("length:"+rec.length());*/
		/*Conto quanti numeri ci sono
		for(int i=0; i<rec.length();i++) {if(rec.charAt(i) == '+') {count++;}}
System.out.println("count:"+count);

		String[] subrec = new String[count];
		for(int j=1; j<=count; j++) {

			subrec[j] = rec.substring(PHONE_NUMBER_LENGTH*(j-1), PHONE_NUMBER_LENGTH*j-1);

		}
		return subrec;
	}*/

	/**
	* Checks if the number is correct
	*
	* Grammar:
	*  full_number = + number | number
	*  number = digit | number
	*  digit = 0 | 1 | ... | 9
	*/
	private boolean isNumberCorrect(String number) {
		int index = 0;
		if (number.charAt(0) == '+') {
			index = 1;
		}
		for (int i = index; i < number.length(); i++) {
				if (!Character.isDigit(number.charAt(i))) {return false;}
		}
		return true;
	}//isNumberCorrect
}//Checker