package alice.tucsonx.sms;

import alice.logictuple.*;
import alice.tucson.api.*;
import org.jsmsengine.*;

/**
 * La classe rappresenta l'agente SMS_Sender
 *
 * @author Mattia Bargellini
 * @version 1.1
 */
public class SMS_Sender extends Agent {

	/**Id per il tc*/
	private TupleCentreId tid;

	/**Creo un oggetto CService*/
	private CService service;

	/**Rappresenta lo stato del servizio*/
	String status;

	/**
	 * Costruttore dell'agente SMS_Sender
	 *
	 * @param id     Id dell'agente
	 * @param target Id del centro di tuple su cui l'agente nasce
 	 */
	public SMS_Sender(AgentId id, TupleCentreId target)
		throws TucsonException {
		super(id);
		tid = target;
	}
	/**
	 * Corpo dell'agente
 	 */
	protected void body() {

		try {


			service  = new CService("com1",9600);

			service.initialize();
			service.setCacheDir(".\\");

			//service.setSmscNumber("");



			while (true) {

				/*Creo il template della tupla da leggere*/
				LogicTuple tosend =
					new LogicTuple(
						"sms_to_send",
						new Var("Id"),
						new Var("Recipient"),
						new Var("Text"));

				/*Eseguo la in da tc*/
				LogicTuple sms = in(tid, tosend);

				/*Estraggo i valori dalla tupla*/
				String id = sms.getArg(0).toString();
				String to = sms.getArg(1).toString();
				String text = sms.getArg(2).toString();

				System.out.println("\nNew SMS to send");
				System.out.println("Id: "+id);
				System.out.println("Recipient: "+to);
				System.out.println("Text: "+text);

				int length = to.length();
				String tonew = to.substring(1, length - 1);

				String newid = id.substring(1,id.length()-1);
				//System.out.println("id length:"+id.length()+"\nnewid:"+newid + "\n");

				//System.out.println(tonew.charAt(0)+"\t");

				// fake service
				if (service==null){

					/*Creo il template della tupla da scrivere*/
					LogicTuple towrite =
							new LogicTuple(
								"sms_service_status",
								new Value(newid),
								new Value("null service"));

					System.out.println("Trying to send sms...\nService status: "+status);

					/*Eseguo la out sul tc*/
					out(tid, towrite);


				} else {


						/*Se la comunicazione  inizializzata allora si eseguono
						 *le operazione descritte nella doc di jSMSEngine,
						 *altrimenti si reinizializza
						 **/
						boolean initialized = service.getInitialized();
						if (!initialized){
							service.initialize();
							service.setCacheDir(".\\");
						}
System.out.println("Trying to send sms...");
						linker(tonew, text);

						/*Creo il template della tupla da scrivere*/
						LogicTuple towrite =
								new LogicTuple(
									"sms_service_status",
									new Value(newid),
									new Value(status));

						System.out.println("Service status: "+status);

						/*Eseguo la out sul tc*/
						out(tid, towrite);
					}


			} //while


		} catch (Exception ex) {
			System.out.println("An exception has occurred: " + ex);
			ex.printStackTrace();
		}

	} //body

	/**Esegue le operazioni di collegamento al cellulare
	 * @param to destinatario
	 * @param text testo da inviare
	 * @return il tipo di errore occorso*/

	private String linker(String to, String text) {
		int temp_err, error;

		/* Operazioni di collegamento col cellulare:
		 *
		 *1. Call initialize() to setup the service.(gi eseguita nel metodo body!)
		 *2. Call connect() to connect with the GSM device.
		 *3. Call sendMessage(), or readMessages() to send or receive messages
		 *   from the device. Call deleteMessage() to delete a message from
		 *   the device's memory.
		 *4. Call refreshDeviceInfo() to get updated GSM device specific information.
		 *5. Call disconnect() to disconnect from the GSM device.
		 */


		temp_err = service.connect();

		if (service.getConnected()) {
			service.setSmscNumber("");
			temp_err = service.sendMessage(new COutgoingMessage(to, text));

			if (temp_err == 0) {
				error = service.disconnect();
			} else {
				error = temp_err;
				service.disconnect();
			}
		} else {
			error = temp_err;
		}

		//CService.refreshDeviceInfo();

		/*Setto il valore dello stato per ogni errore riscontrato*/
		switch (error) {
			case 0 :
				status = "OK";
				break;
			case -1 :
				status = "Generic Error";
				break;
			case -10 :
				status = "Not initialized";
				break;
			case -11 :
				status = "Not connected";
				break;
			case -20 :
				status = "Device does not support ASCII or PDU mode";
				break;
			case -21 :
				status = "Charset hex not supported";
				break;
			case -22 :
				status = "Cannot disable indications";
				break;
			case -30 :
				status = "Message not found";
				break;
			case -40 :
				status = "Send-message operation failed";
				break;
			case -100 :
				status = "Invalid dir";
				break;
			case -101 :
				status = "No cache";
				break;
			case -102 :
				status = "the PIN given is invalid";
				break;
			case -9999 :
				status =
					"The specified operation is not supported by jSMSEngine API.";
				break;

			default :
				status = "Unknown Error" + " " + error;
				break;
		}

		return status;

	} //linker

}