package alice.tucsonx.sms;

import alice.tucson.api.*;
import alice.util.Tools;
import alice.logictuple.*;

/**
 * La classe lancia il Servizio
 *
 * @author Mattia Bargellini
 * @version 1.0
 */
public class StartAlarm
{
    /**Centro di tuple di default*/
    private final static String DEFAULT_TID = "sms";
    /**Identificativo di default dell'agente*/
    private final static String DEFAULT_AID = "alarm_clock";


    public static void main(String[] args) throws Exception
    {
    	if (Tools.isOpt(args,"-help")||Tools.isOpt(args,"-?")){
    		System.out.println("Arguments: {-tid <Tuple Centre Id>} {-aid <Agent Id>}");
			System.out.println(" -tid option specifies the identifier of the tuple centre used for the service (default is sms).");
			System.out.println(" -aid option specifies the identifier of the agent used for the service (default is alarm_clock).");
    		System.exit(-1);
    	}

		System.out.println("Starting Alarm service...");

        TupleCentreId tid = new TupleCentreId(DEFAULT_TID);
        AgentId aid = new AgentId(DEFAULT_AID);

        String stid = Tools.getOpt(args,"-tid");
        if (stid!=null) {
        	try {
        		tid = new TupleCentreId(stid);
        	} catch (Exception ex){
        		System.err.println("Invalid Tuple Centre Identifier "+stid+". Using the default (sms).");
        	}
        }

		String said = Tools.getOpt(args,"-aid");
		if (stid!=null) {
			try {
				aid = new AgentId(said);
			} catch (Exception ex){
				System.err.println("Invalid Agent Identifier "+aid+". Using the default (alarm_clock).");
			}
		}




		System.out.println("Configuring tuple centre "+tid+"...");

     	String spec =
     		"reaction(out(reset),(\n"+
  				"in_r(reset),\n"+
  				"out_r(reset_loop) )).\n\n"+
			"reaction(out_r(reset_loop),(\n"+
  				"in_r(sms(Id,Rec,Text,Data,Ora)),\n"+
  				"out_r(temp(Id,Rec,Text)),\n"+
  				"out_r(when(Id,Data,Ora)),\n"+
  				"out_r(reset_loop) )).\n\n"+
			"reaction(out_r(reset_loop),(\n"+
 				 "in_r(reset_loop) )).\n\n"+
     		"reaction(out(sms(Id,Recipient,Text,Date,Time)),(\n"+
     			"in_r(sms(Id,Recipient,Text,Date,Time)),\n"+
     			"out_r(temp(Id,Recipient,Text)),\n"+
     			"out_r(when(Id,Date,Time)) )).\n\n"+
			"reaction(out(error(Id,Texterror)),(\n"+
				"in_r(temp(Id,_,_)) )).\n\n"+
			"reaction(out(wake_up(Id)),(\n"+
				"in_r(temp(Id,Recipient,Text)),\n"+
				"out_r(sms(Id,Recipient,Text)) )).\n\n";

		TucsonContext ct = Tucson.enterDefaultContext();
		spec += ct.getSpec(tid);
		ct.setSpec(tid,spec);

		/*while (ct.inp(tid, new LogicTuple("ticket_counter",new Var()))!=null){}*/

		ct.out(tid,new LogicTuple("reset"));

		System.out.println("Tuple centre "+tid+" configured.");
		System.out.println("Spawning the AlarmClock agent..");
        new AlarmClock(aid,tid).spawn();
        System.out.println("Agent spawn.");
		System.out.println("AlarmClock ready.");
    }
}

