package alice.tucsonx.sms;

import alice.tucson.api.*;
import alice.util.Tools;
import alice.logictuple.*;

import org.jsmsengine.*;
/**
 * La classe lancia il Servizio
 *
 * @author Mattia Bargellini
 * @version 1.0
 */
public class StartService
{
    /**Centro di tuple di default*/
    private final static String DEFAULT_TID = "sms";
    /**Identificativo di default dell'agente*/
    private final static String DEFAULT_AID = "sms_sender";


    public static void main(String[] args) throws Exception
    {
    	if (Tools.isOpt(args,"-help")||Tools.isOpt(args,"-?")){
    		System.out.println("Arguments: {-tid <Tuple Centre Id>} {-aid <Agent Id>}");
			System.out.println(" -tid option specifies the identifier of the tuple centre used for the service (default is sms).");
			System.out.println(" -aid option specifies the identifier of the agent used for the service (default is sms_sender).");
    		System.exit(-1);
    	}

		System.out.println("Starting SMS service...");

        TupleCentreId tid = new TupleCentreId(DEFAULT_TID);
        AgentId aid = new AgentId(DEFAULT_AID);

        String stid = Tools.getOpt(args,"-tid");
        if (stid!=null) {
        	try {
        		tid = new TupleCentreId(stid);
        	} catch (Exception ex){
        		System.err.println("Invalid Tuple Centre Identifier "+stid+". Using the default (sms).");
        	}
        }

		String said = Tools.getOpt(args,"-aid");
		if (stid!=null) {
			try {
				aid = new AgentId(said);
			} catch (Exception ex){
				System.err.println("Invalid Agent Identifier "+aid+". Using the default (sms_sender).");
			}
		}


		System.out.println("Configuring tuple centre "+tid+"...");
/**/
     	String spec =
     		"reaction(out(reset_sms),(\n"+
  				"in_r(reset_sms),\n"+
  				"out_r(reset_loop_sms) )).\n\n"+
			"reaction(out_r(reset_loop_sms),(\n"+
  				"in_r(sms(Id,Rec,Text)),\n"+
  				"out_r(smsx(Id,Rec,Text)),\n"+
  				"out_r(reset_loop_sms) )).\n\n"+
			"reaction(out_r(reset_loop_sms),(\n"+
 				 "in_r(reset_loop_sms) )).\n\n"+
     		"reaction(out(sms(Id,Recipient,Text)),(\n"+
			"	in_r(sms(Id,Recipient,Text)),\n"+
			"	spawn(checker,java('alice.tucsonx.sms.Checker'),[sms(Id,Recipient,Text)]) )).\n\n"+
     		"reaction(out_r(smsx(Id,Recipient,Text)),(\n"+
			"	in_r(smsx(Id,Recipient,Text)),\n"+
			"	spawn(checker,java('alice.tucsonx.sms.Checker'),[smsx(Id,Recipient,Text)]) )).\n\n"+
			"reaction(out_r(sms(Id,Recipient,Text)),(\n"+
			"	in_r(sms(Id,Recipient,Text)),\n"+
			"	spawn(checker,java('alice.tucsonx.sms.Checker'),[sms(Id,Recipient,Text)]) )).\n\n"+
     		"reaction(in(ticket(Id)),(\n"+
			"  pre,\n"+
			"  in_r(ticket_counter(Id)),\n"+
			"  Id1 is Id + 1,\n"+
			"  out_r(ticket_counter(Id1)),\n"+
			"  out_r(ticket(Id)) )).\n\n";

		TucsonContext ct = Tucson.enterDefaultContext();
		spec += ct.getSpec(tid);
		ct.setSpec(tid,spec);

		while (ct.inp(tid, new LogicTuple("ticket_counter",new Var()))!=null){}
		ct.out(tid,new LogicTuple("ticket_counter", new Value(0)));

		ct.out(tid,new LogicTuple("reset_sms"));

		System.out.println("Tuple centre "+tid+" configured.");
		System.out.println("Spawning the SMS agent..");
        new SMS_Sender(aid,tid).spawn();
        System.out.println("Agent spawn.");
		System.out.println("SMS ready.");
    }
}